@@ -293,25 +293,40 @@ class Experiment(ABC):
         if AOsignalPath.endswith(".cdh"):
             with open(AOsignalPath, "r") as file:
                 cdh_content = file.readlines()
+            
+            cdf_path = AOsignalPath.replace(".cdh", ".cdf")
 
-            # Extraction des paramètres
-            n_events = int([line.split(":")[1].strip() for line in cdh_content if "Number of events" in line][0])
-            n_acquisitions = int([line.split(":")[1].strip() for line in cdh_content if "Number of acquisitions per event" in line][0])
+            # Extraire les paramètres depuis le fichier .cdh
+            n_scans = int([line.split(":")[1].strip() for line in cdh_content if "Number of events" in line][0])
+            n_acquisitions_per_event = int([line.split(":")[1].strip() for line in cdh_content if "Number of acquisitions per event" in line][0])
             num_elements = int([line.split(":")[1].strip() for line in cdh_content if "Number of US transducers" in line][0])
 
-            # Initialisation avec la bonne forme : (n_acquisitions, n_events)
-            AOsignal_matrix = np.zeros((n_acquisitions, n_events), dtype=np.float32)
+            # Initialisation des structures
+            AO_signal = np.zeros((n_acquisitions_per_event, n_scans), dtype=np.float32)
+            active_lists = []
+            angles = []
 
-            with open(AOsignalPath.replace(".cdh", ".cdf"), "rb") as file:
-                hex_length = (num_elements + 3) // 4
-                for event in range(n_events):
-                    file.read(hex_length // 2)  # Sauter l'en-tête de l'événement
-                    signal = np.frombuffer(file.read(n_acquisitions * 4), dtype=np.float32)
-                    AOsignal_matrix[:, event] = signal  # Remplir par colonne
+            # Lecture du fichier binaire
+            with open(cdf_path, "rb") as file:
+                for j in trange(n_scans, desc="Lecture des événements"):
+                    # Lire l'activeList : 48 caractères hex = 24 bytes
+                    active_list_bytes = file.read(24)
+                    active_list_hex = active_list_bytes.hex()
+                    active_lists.append(active_list_hex)
 
-            AOsignal_matrix = AOsignal_matrix
+                    # Lire l'angle (1 byte signé)
+                    angle_byte = file.read(1)
+                    angle = np.frombuffer(angle_byte, dtype=np.int8)[0]
+                    angles.append(angle)
+
+                    # Lire le signal AO (float32)
+                    data = np.frombuffer(file.read(n_acquisitions_per_event * 4), dtype=np.float32)
+                    if len(data) != n_acquisitions_per_event:
+                        raise ValueError(f"Erreur à l'événement {j} : attendu {n_acquisitions_per_event}, obtenu {len(data)}")
+                    AO_signal[:, j] = data
+
+            return AO_signal
 
-            return AOsignal_matrix
 
         elif AOsignalPath.endswith(".npy"):
             return np.load(AOsignalPath)  # Supposé déjà au bon format
@@ -348,7 +363,7 @@ class Experiment(ABC):
             f"Number of acquisitions per event: {AO_signal.shape[0]}\n"
             f"Start time (s): 0\n"
             f"Duration (s): 1\n"
-            f"Acquisition frequency (Hz): {1/self.AcousticFields[0].kgrid.dt}\n"
+            f"Acquisition frequency (Hz): {self.params.acoustic['f_saving']}\n"
             f"Data mode: histogram\n"
             f"Data type: AOT\n"
             f"Number of US transducers: {self.params.acoustic['num_elements']}"
