@@ -66,8 +66,10 @@ class BayesianRecon(AlgebraicRecon):
             dir_name += f'_Beta_{self.beta}_Sigma_{self.sigma}'
 
         results_dir = os.path.join(self.saveDir, dir_name)
+        if not os.path.exists(results_dir):
+            os.makedirs(results_dir)
 
-        if os.path.exists(os.path.join(results_dir,"reconIndices.npy")):
+        if os.path.exists(os.path.join(results_dir,"indices.npy")):
             return (True, results_dir)
 
         return (False, results_dir)
