@@ -60,7 +60,17 @@ class OptimizerType(Enum):
     This optimizer is compatible with both histogram and list-mode data.
     This optimizer is compatible with both emission and transmission data.
     """
-    LS_TV = 'LS_TV'
+    CP_TV = 'CP_TV'
+    """
+    This optimizer implements the Chambolle-Pock algorithm for total variation regularization.
+    It is suitable for problems where the objective function includes a total variation term.
+    It is particularly effective for preserving edges while reducing noise in the reconstructed image.
+    """ 
+    CP_KL = 'CP_KL'
+    """
+    This optimizer implements the Kullback-Leibler divergence for regularization.
+    It is suitable for problems where the objective function includes a Kullback-Leibler divergence term.
+    """
     LS = 'LS'
     """
     This optimizer implements the standard Landweber algorithm for least-squares optimization.
